﻿using System;
using System.ComponentModel;
using System.Windows;
using System.Windows.Markup;
using System.Windows.Media;
using System.Windows.Media.Imaging;

namespace TestProject
{
    [ContentProperty("Gradients")]
    [MarkupExtensionReturnType(typeof(ImageBrush))]
    public class CustomBrush : MarkupExtension, INotifyPropertyChanged
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Event
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 속성 변경시 이벤트 - PropertyChanged

        /// <summary>
        /// 속성 변경시 이벤트
        /// </summary>
        public event PropertyChangedEventHandler PropertyChanged;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 그라디언트 중단 컬렉션
        /// </summary>
        private GradientStopCollection gradientStopCollection = null;

        /// <summary>
        /// 그라디언트 크기
        /// </summary>
        private int gradientSize = 500;

        /// <summary>
        /// 그라디언트 디테일
        /// </summary>
        private double gradientDetail = 10;

        /// <summary>
        /// 중심 포인트
        /// </summary>
        private Point centerPoint = new Point(0.5, 0.5);

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Property
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 그라디언트 중단 컬렉션 - Gradients

        /// <summary>
        /// 그라디언트 중단 컬렉션
        /// </summary>
        public GradientStopCollection Gradients
        {
            get
            {
                return this.gradientStopCollection;
            }
            set 
            {
                this.gradientStopCollection = value;

                this.gradientStopCollection.Changed += gradientStopCollection_Changed;

                FirePropertyChangedEvent("Gradients"); 
            }
        }

        #endregion
        #region 그라디언트 크기 - GradientSize

        /// <summary>
        /// 그라디언트 크기
        /// </summary>
        public int GradientSize
        {
            get
            {
                return this.gradientSize;
            }
            set
            {
                this.gradientSize = value;
                
                FirePropertyChangedEvent("GradientSize");
            }
        }

        #endregion
        #region 그라디언트 디테일 - GradientDetail

        /// <summary>
        /// 그라디언트 디테일
        /// </summary>
        public double GradientDetail
        {
            get
            {
                return this.gradientDetail;
            }
            set
            {
                this.gradientDetail = value;
                
                FirePropertyChangedEvent("GradientDetail");
            }
        }

        #endregion
        #region 중심 포인트 - CenterPoint

        /// <summary>
        /// 중심 포인트
        /// </summary>
        public Point CenterPoint
        {
            get
            {
                return this.centerPoint;
            }
            set
            {
                this.centerPoint = value;
                
                FirePropertyChangedEvent("CenterPoint");
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 타겟 객체 - TargetObject

        /// <summary>
        /// 타겟 객체
        /// </summary>
        private DependencyObject TargetObject { get; set; }

        #endregion
        #region 타겟 속성 - TargetProperty

        /// <summary>
        /// 타겟 속성
        /// </summary>
        private DependencyProperty TargetProperty { get; set; }

        #endregion
        #region 기본 브러시 - BaseBrush

        /// <summary>
        /// 기본 브러시
        /// </summary>
        private ImageBrush BaseBrush { get; set; }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - CustomBrush()

        /// <summary>
        /// 생성자
        /// </summary>
        public CustomBrush()
        {
            BaseBrush = new ImageBrush();
            Gradients = new GradientStopCollection();
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 값 제공하기 - ProvideValue(serviceProvider)

        /// <summary>
        /// 값 제공하기
        /// </summary>
        /// <param name="serviceProvider">서비스 공급자</param>
        /// <returns>값</returns>
        public override object ProvideValue(IServiceProvider serviceProvider)
        {
            UpdateBrush();

            return BaseBrush;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Protected
        //////////////////////////////////////////////////////////////////////////////// Function

        #region 그라데이션 색상 배열 구하기 - GetGradationColorArray()

        /// <summary>
        /// 그라데이션 색상 배열 구하기
        /// </summary>
        /// <returns>그라데이션 색상 배열</returns>
        protected int[] GetGradationColorArray()
        {
            int gradationCount = (int)(360 / GradientDetail);

            int[] gradationColorArray = new int[gradationCount];

            DrawingVisual DrawingVisual = new DrawingVisual();

            DrawingContext DrawingContext = DrawingVisual.RenderOpen();

            LinearGradientBrush linearGradientBrush = new LinearGradientBrush(Gradients)
            {
                StartPoint = new Point(0, 0),
                EndPoint   = new Point(1, 0)
            };

            DrawingContext.DrawRectangle(linearGradientBrush, null, new Rect(0, 0, gradationCount, 1));

            DrawingContext.Close();

            RenderTargetBitmap renderTargetBitmap = new RenderTargetBitmap(gradationCount, 1, 96, 96, PixelFormats.Pbgra32);

            renderTargetBitmap.Render(DrawingVisual);           

            renderTargetBitmap.CopyPixels(new Int32Rect(0, 0, gradationCount, 1), gradationColorArray, gradationCount * 4, 0);

            return gradationColorArray;
        }
        
        #endregion
        #region 브러시 업데이트하기 - UpdateBrush()

        /// <summary>
        /// 브러시 업데이트하기
        /// </summary>
        protected void UpdateBrush()
        {
            int size = GradientSize * 2;

            Point actualCenterPoint = new Point(size * CenterPoint.X, size * CenterPoint.Y);

            int[,] pixelArray = new int[size, size];

            int[] gradationColorArray = GetGradationColorArray();

            for(int x = 0; x < size; x++)
            {
                for(int y = 0; y < size; y++)
                {
                    double angle = Math.Atan2(y - actualCenterPoint.X, x - actualCenterPoint.Y) * 180 / Math.PI;

                    int colorIndex = (int)(angle / GradientDetail);

                    if(colorIndex < 0)
                    {
                        colorIndex = gradationColorArray.Length + colorIndex;
                    }

                    int color = gradationColorArray[colorIndex];

                    pixelArray[x, y] = color;
                }
            }

            WriteableBitmap writeableBitmap = new WriteableBitmap(size, size, 96, 96, PixelFormats.Pbgra32, null);

            writeableBitmap.WritePixels(new Int32Rect(0, 0, size, size), pixelArray, (int)writeableBitmap.Width * 4, 0);

            writeableBitmap.Freeze();

            BaseBrush.ImageSource = writeableBitmap;
        }

        #endregion

        #region 속성 변경시 이벤트 발생시키기 - FirePropertyChangedEvent(propertyName)

        /// <summary>
        /// 속성 변경시 이벤트 발생시키기
        /// </summary>
        /// <param name="propertyName">속성명</param>
        protected void FirePropertyChangedEvent(string propertyName)
        {
            UpdateBrush();

            if(PropertyChanged != null)
            {
                PropertyChanged(this, new PropertyChangedEventArgs(propertyName));
            }
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 그라디언트 중단 컬렉션 변경시 처리하기 - gradientStopCollection_Changed(sender, e)

        /// <summary>
        /// 그라디언트 중단 컬렉션 변경시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void gradientStopCollection_Changed(object sender, EventArgs e)
        {
            FirePropertyChangedEvent("GradientDetail");
        }

        #endregion
    }
}