﻿using System.Reflection;
using System.Windows;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Shapes;

namespace TestProject
{
    /// <summary>
    /// 메인 윈도우
    /// </summary>
    public partial class MainWindow : Window
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - MainWindow()

        /// <summary>
        /// 생성자
        /// </summary>
        public MainWindow()
        {
            InitializeComponent();

            this.rectangle.MouseDown += rectangle_MouseDown;
            this.addButton.Click     += addButton_Click;
            this.deleteButton.Click  += deleteButton_Click;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Private
        //////////////////////////////////////////////////////////////////////////////// Event

        #region 사각형 마우스 DOWN 처리하기 - rectangle_MouseDown(sender, e)

        /// <summary>
        /// 사각형 마우스 DOWN 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void rectangle_MouseDown(object sender, MouseButtonEventArgs e)
        {
            Rectangle rectangle = sender as Rectangle;

            Point mousePoint = e.GetPosition(rectangle);

            Point centerPoint = new Point(mousePoint.X / rectangle.ActualWidth, mousePoint.Y / rectangle.ActualHeight);

            this.customBrush.CenterPoint = centerPoint;
        }

        #endregion
        #region 추가 버튼 클릭시 처리하기 - addButton_Click(sender, e)

        /// <summary>
        /// 추가 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void addButton_Click(object sender, RoutedEventArgs e)
        {
            if(this.colorComboBox.SelectedItem != null)
            {
                this.colorListBox.Items.Add(this.colorComboBox.SelectedItem);

                UpdateGradientOffset();
            }
        }

        #endregion
        #region 삭제 버튼 클릭시 처리하기 - deleteButton_Click(sender, e)

        /// <summary>
        /// 삭제 버튼 클릭시 처리하기
        /// </summary>
        /// <param name="sender">이벤트 발생자</param>
        /// <param name="e">이벤트 인자</param>
        private void deleteButton_Click(object sender, RoutedEventArgs e)
        {
            if(this.colorListBox.SelectedItem != null)
            {
                this.colorListBox.Items.Remove(this.colorListBox.SelectedItem);

                UpdateGradientOffset();
            }
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////// Function

        #region 그라디언트 오프셋 업데이트하기 - UpdateGradientOffset()

        /// <summary>
        /// 그라디언트 오프셋 업데이트하기
        /// </summary>
        private void UpdateGradientOffset()
        {
            double offset = 1.0 / (this.colorListBox.Items.Count - 1);

            if(offset == double.PositiveInfinity)
            {
                offset = 1;
            }

            GradientStopCollection gradientStopCollection = new GradientStopCollection();

            for(int i = 0; i < this.colorListBox.Items.Count; i++)
            {
                Color Color = (Color)ColorConverter.ConvertFromString((this.colorListBox.Items[i] as PropertyInfo).Name);

                gradientStopCollection.Add(new GradientStop() { Color = Color, Offset = offset * i });
            }
          
            this.customBrush.Gradients = gradientStopCollection;
        }

        #endregion
    }
}